; nPoly - Polygonal Collision Library
; By Jocelyn 'GoSsE' Perreault, http://www.nuloen.com/
; Last Updated: July 8 2004
; Version: 1.0.0

; This source code is copyrighted (c) to Jocelyn Perreault.
; It is FreeWare. You have no limitiations in regard to 
; applications/games that you release/create with it, either
; they be freeware or commercial. However, you cannot release
; this source code under another name or release a similar
; library with it. It cannot be redistributed. The code may be 
; modified to be tweaked but you cannot distribute it. If you
; made a modification that you think the community would
; benifit, please let me know.
; If you make tons of money with a commercial project,
; a free copy wouldn't hurt ;)
; I would love to have an e-mail and a place in the credits
; of your projects, but it is up to you.
; Thanks,
;                 Jocelyn "GoSsE Korupted" Perreault
;                  gosse@nuloen.com
;                  Nuclear Loaded Entertainment
;                  http://www.nuloen.com/

; ======================================================================
; Globals and Constants
; ======================================================================
Const nP_MAX = 32			; Maximum number of vertices for a poly

; ======================================================================
; Different objects
; ======================================================================

; Polygon
Type nP_Poly
	Field iX[nP_MAX]		; Vertices X position
	Field iY[nP_MAX]		; Vertices Y position
	Field fX#[nP_MAX]		; Scaled and rotated X position
	Field fY#[nP_MAX]		; Scaled and rotated Y position
	Field iVertexCount		; Number of vertices
	Field fScaleX#			; X-Scale
	Field fScaleY#			; Y-Scale
	Field fAngle#			; Angle rotation
	Field iMinX				; Bounding box
	Field iMinY				; Bounding box
	Field iMaxX				; Bounding box
	Field iMaxY				; Bounding box
End Type

; ======================================================================
; Functions
; ======================================================================

; Creates a new empty polygon
Function nP_CreatePoly()
	Local nPoly.nP_Poly = New nP_Poly
	nPoly\fScaleX = 1
	nPoly\fScaleY = 1
	Return Handle(nPoly)
End Function

; ----------------------------------------------------------------------

; Adds a vertex to a polygon
Function nP_AddVertex(poly, x, y, bUpdate = True)
	Local nPoly.nP_Poly = Object.nP_Poly(poly)
	If nPoly\iVertexCount > nP_MAX Then
		RuntimeError "Cannot add one more vertex to polygon!"
	Else
		nPoly\iX[nPoly\iVertexCount] = x
		nPoly\iY[nPoly\iVertexCount] = y
		nPoly\iVertexCount = nPoly\iVertexCount + 1
		If bUpdate Then
			nP_UpdatePoly(poly)
		End If
	End If
End Function

; ----------------------------------------------------------------------

; Checks if 2 polygons overlap each other
Function nP_PolyOverlap(poly1, x1, y1, poly2, x2, y2, bCheckBounding = True)
	; Polygons
	Local nPoly1.nP_Poly = Object.nP_Poly(poly1)
	Local nPoly2.nP_Poly = Object.nP_Poly(poly2)
	; Temp positions
	Local tx1#, ty1#, tx2#, ty2#, tx3#, ty3#, tx4#, ty4#
	; Delta calculations
	Local dSqr#, d1#, d2#, dx#, dy#
	; Last vertex
	Local lx1#, ly1#, lx2#, ly2#
	; Counters
	Local i, j
	; Checks bounding box first (if specified)
	If bCheckBounding Then
		tx1 = X1 + nPoly1\iMinX
		ty1 = Y1 + nPoly1\iMinY
		tx2 = X1 + nPoly1\iMaxX
		ty2 = Y1 + nPoly1\iMaxY
		tx3 = X2 + nPoly2\iMinX
		ty3 = Y2 + nPoly2\iMinY
		tx4 = X2 + nPoly2\iMaxX
		ty4 = Y2 + nPoly2\iMaxX
		If Not (tx3 > tx1 And ty3 > ty1 And tx3 < tx2 And ty3 < ty2) Then
			If Not (tx4 > tx1 And ty4 > ty1 And tx4 < tx2 And ty4 < ty2) Then
				If Not (tx3 > tx1 And ty4 > ty1 And tx3 < tx2 And ty4 < ty2) Then
					If Not (tx4 > tx1 And ty3 > ty1 And tx4 < tx2 And ty3 < ty2) Then
						Return False
					End If
				End If
			End If
		End If
	End If
	; Store the last vertex
	lx1 = nPoly1\fX#[0] + X1
	ly1 = nPoly1\fY#[0] + Y1
	lx2 = nPoly2\fX#[0] + X2
	ly2 = nPoly2\fY#[0] + Y2
	; Cycle through all other vertices
	For i = 1 To nPoly1\iVertexCount - 1
		For j = 1 To nPoly2\iVertexCount - 1
			; Store vertices
			tx1 = nPoly1\fX#[i] + X1
			ty1 = nPoly1\fY#[i] + Y1
			tx3 = nPoly2\fX#[j] + X2
			ty3 = nPoly2\fY#[j] + Y2
			tx2 = lX1
			ty2 = lY1
			tx4 = lX2
			ty4 = lY2
			; Calculate deltas
			dSqr = (ty4-ty3)*(tx2-tx1)-(tx4-tx3)*(ty2-ty1)
			d1 = (ty1-ty3)
			d2 = (tx1-tx3)
			dX = ((tx2-tx1)*d1-(ty2-ty1)*d2)
			dY = ((tx4-tx3)*d1-(ty4-ty3)*d2)
			; Parallel
			If dSqr = 0 Then
				; Coincident
				If dX = 0 And dY = 0 Then
					Return True
				End If
			Else
				; Otherwise
				d1 = dX / dSqr
				d2 = dY / dSqr
				If (d1 >= 0 And d1 <= 1) And (d2 >= 0 And d2 <= 1) Then
					Return True
				End If
			End If
			; Store last vertex
			lX2 = tx3
			lY2 = ty3
		Next
		; Store last vertex
		lX1 = tx1
		lY1 = ty1
	Next
	Return False
End Function

; ----------------------------------------------------------------------

Function nP_PolyOverlapsCircle(poly, X, Y, iCircleX, iCircleY, iCircleRadius, bCheckBounding = True)
	Local nPoly.nP_Poly = Object.nP_Poly(poly)
	; Temp positions
	Local tx1#, ty1#, tx2#, ty2#, tx3#, ty3#, tx4#, ty4#
	Local CX1#, CY1#, CX2#, CY2#, pX1#, pY1#, pX2#, pY2#, pXT#, pYT#
	; Delta calculations
	Local d1#, d2#, dX#, dY#, dSqr#
	; Checks bounding box first (if specified)
	If bCheckBounding Then
		tx1 = X + nPoly\iMinX
		ty1 = Y + nPoly\iMinY
		tx2 = X + nPoly\iMaxX
		ty2 = Y + nPoly\iMaxY
		tx3 = iCircleX - iCircleRadius
		ty3 = iCircleY - iCircleRadius
		tx4 = iCircleX + iCircleRadius
		ty4 = iCircleY + iCircleRadius
		If Not (tx3 > tx1 And ty3 > ty1 And tx3 < tx2 And ty3 < ty2) Then
			If Not (tx4 > tx1 And ty4 > ty1 And tx4 < tx2 And ty4 < ty2) Then
				If Not (tx3 > tx1 And ty4 > ty1 And tx3 < tx2 And ty4 < ty2) Then
					If Not (tx4 > tx1 And ty3 > ty1 And tx4 < tx2 And ty3 < ty2) Then
						Return False
					End If
				End If
			End If
		End If
	End If
	; Store the last vertex & circle
	lx = nPoly\fX#[0] + X
	ly = nPoly\fY#[0] + Y
	CX2# = iCircleX
	CY2# = iCircleY
	; Cycle through all other vertices
	For i = 1 To nPoly\iVertexCount - 1	
		CX1# = nPoly\fX#[i] + X
		CY1# = nPoly\fY#[i] + Y
		pX1 = lX - CX1
		pY1 = lY - CY1
		pX2 = CX2 - CX1
		pY2 = CY2 - CY1
		d1 = pX1 * pX2 + pY1 * pY2
		If d1 <= 0 Then
			dX# = CX2 - CX1
			dY# = CY2 - CY1
			dSqr# = dX * dX + dY * dY
		Else
			d2 = pX1 * pX1 + pY1 * pY1
			If (d2 <= d1) Then
				dX# = CX2 - lX
				dY# = CY2 - lY
				dSqr# = dX * dX + dY * dY
			Else
				d1 = d1 / d2
				pXT# = CX1 + d1 * pX1
				pYT# = CY1 + d1 * pY1
				dX# = CX2 - pXT
				dY# = CY2 - PYT
				dSqr# = dX * dX + dY * dY
			End If
		End If
		If dSqr# < iCircleRadius * iCircleRadius Then
			Return True
		End If
		lx = CX1
		ly = CY1
	Next
	Return False	
End Function

; ----------------------------------------------------------------------

; Returns true if both circles overlap
Function nP_CirclesOverlap(x1, y1, rad1, x2, y2, rad2)
	Local dx# = x2 - x1
	Local dy# = y2 - y1
	Local rsqr = rad1 + rad2
	rsqr = rsqr*rsqr
	Return (dx*dx+dy*dy < rsqr)
End Function

; ----------------------------------------------------------------------

; Scales the polygon by the specified scale
Function nP_ScalePoly(poly, sx#, sy#, bUpdate = True)
	Local nPoly.nP_Poly = Object.nP_Poly(poly)
	nPoly\fScaleX# = nPoly\fScaleX# * sx#
	nPoly\fScaleY# = nPoly\fScaleY# * sy#
	If bUpdate Then
		nP_UpdatePoly(poly)
	End If
End Function

; ----------------------------------------------------------------------

; Resizes the polygon to the specieid scale
Function nP_ResizePoly(poly, sx#, sy#, bUpdate = True)
	Local nPoly.nP_Poly = Object.nP_Poly(poly)
	nPoly\fScaleX# = sx#
	nPoly\fScaleY# = sy#
	If bUpdate Then
		nP_UpdatePoly(poly)
	End If
End Function

; ----------------------------------------------------------------------

; Rotates the polygon by the specified rotation angle
Function nP_RotatePoly(poly, fRot#, bUpdate = True)
	Local nPoly.nP_Poly = Object.nP_Poly(poly)
	nPoly\fAngle# = nPoly\fAngle# + fRot#
	If bUpdate Then
		nP_UpdatePoly(poly)
	End If
End Function

; ----------------------------------------------------------------------

; Turns the polygon to the specified angle
Function nP_FreeRotatePoly(poly, fAngle#, bUpdate = True)
	Local nPoly.nP_Poly = Object.nP_Poly(poly)
	nPoly\fAngle# = fAngle#
	If bUpdate Then
		nP_UpdatePoly(poly)
	End If
End Function

; ----------------------------------------------------------------------

; Updates the polygon according to scale and rotation
Function nP_UpdatePoly(poly)
	Local nPoly.nP_Poly = Object.nP_Poly(poly)
	Local fX#, fY#, f1#, f2#
	Local i
	; Set up big extremums
	nPoly\iMinX = 9999999
	nPoly\iMinY = 9999999
	nPoly\iMaxX = -9999999
	nPoly\iMaxY = -9999999
	; Find a valid collision setting
	For i = 0 To nPoly\iVertexCount - 1
		; Standard data
		fX = nPoly\iX[i]
		fY = nPoly\iY[i]
		; Scale
		fX = fX * nPoly\fScaleX
		fY = fY * nPoly\fScaleY
		; Rotation
		If nPoly\fAngle# <> 0 Then
			f1 = Cos(nPoly\fAngle#) * fX - Sin(nPoly\fAngle#) * fY
			f2 = Sin(nPoly\fAngle#) * fX + Cos(nPoly\fAngle#) * fY
			fX = f1
			fY = f2
		End If
		; Final Values
		nPoly\fX#[i] = fX
		nPoly\fY#[i] = fY
		; Check for bounding box
		If fX < nPoly\iMinX Then
			nPoly\iMinX = fX
		End If
		If fY < nPoly\iMinY Then
			nPoly\iMinY = fY
		End If
		If fX > nPoly\iMaxX Then
			nPoly\iMaxX = fX
		End If
		If fY > nPoly\iMaxY Then
			nPoly\iMaxY = fY
		End If
	Next
End Function

; ----------------------------------------------------------------------

; Draws the polygon on screen at the specified location
Function nP_DrawPoly(poly, x, y, r=255, g=255, b=255, bDrawBoundingBox=True, br=255, bg=255, bb=255)
	Local nPoly.nP_Poly = Object.nP_Poly(poly)
	Local i, lx, ly
	; Draw bounding box (if specified)
	If bDrawBoundingBox Then
		Color br, bg, bb
		Rect nPoly\iMinX + x, nPoly\iMinY + y, nPoly\iMaxX - nPoly\iMinX + 1, nPoly\iMaxY - nPoly\iMinY + 1, False
	End If
	; Draw all the lines
	Color r, g, b
	lx = nPoly\fX[0] + x
	ly = nPoly\fY[0] + y
	For i = 1 To nPoly\iVertexCount - 1
		Line lx, ly, nPoly\fX[i] + x, nPoly\fY[i] + y
		lx = nPoly\fX[i] + x
		ly = nPoly\fY[i] + y
	Next
End Function

; ----------------------------------------------------------------------

; Copies a polygon and return the newly created poly
Function nP_CopyPoly(poly)
	Local nPoly.nP_Poly = Object.nP_Poly(poly)
	Local newPoly.nP_Poly = New nP_Poly
	Local i
	; Copy point data
	For i = 0 To nPoly\iVertexCount - 1
		newPoly\iX[i] = nPoly\iX[i]
		newPoly\iY[i] = nPoly\iY[i]
		newPoly\fX#[i] = nPoly\fX#[i]
		newPoly\fY#[i] = nPoly\fY#[i]
	Next
	; Copy all data
	newPoly\iVertexCount = nPoly\iVertexCount
	newPoly\fScaleX# = nPoly\fScaleX#
	newPoly\fScaleY# = nPoly\fScaleY#
	newPoly\fAngle# = nPoly\fAngle#
	newPoly\iMinX = nPoly\iMinX
	newPoly\iMinY = nPoly\iMinY
	newPoly\iMaxX = nPoly\iMaxX
	newPoly\iMaxY = nPoly\iMaxY
	Return Handle(newPoly)
End Function

; ----------------------------------------------------------------------

; Deletes the polygon from memory
Function nP_FreePoly(poly)
	Local nPoly.nP_Poly = Object.nP_Poly(poly)
	Delete nPoly
End Function